(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
This is the FeynMaster model file for QED.
Created by: Duarte Fontes
Email: duartefontes@tecnico.ulisboa.pt
Last Update: 30.07.2020
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)

(***** Parameter list *****)
M$Parameters = {
  mf == {TeXName -> "m_f", Renormalization -> {mf -> mf + dmf}},
  ee == {TeXName -> "e", Renormalization -> {ee -> ee + de ee}},
  xiA == {TeXName -> "\\xi_A"},
  de == {Counterterm -> True, TeXName -> "\\delta e"},
  dZ3 == {Counterterm -> True, TeXName -> "\\delta Z_3"},
  dZfL == {Counterterm -> True, TeXName -> "\\delta Z_f^L"},
  dZfR == {Counterterm -> True, TeXName -> "\\delta Z_f^R"},
  dmf == {Counterterm -> True, TeXName -> "\\delta m_f"}};

(***** Gauge group list *****)
M$GaugeGroups = {
  U1EM == {
		Abelian -> True,
		GaugeBoson -> A,
		Charge -> Q,
		CouplingConstant -> ee}};

(***** Particle classes list *****)
M$ClassesDescription = {
  W[1] == {
		ClassName -> psi1L,
		SelfConjugate -> False,
		QuantumNumbers -> {Q-> Q},
		Renormalization -> {psi1L -> psi1L + 1/2 dZfL psi1L},
		Chirality -> Left},
  W[2] == {
		ClassName -> chi1R,
		SelfConjugate -> False,
		QuantumNumbers -> {Q-> Q},
		Renormalization -> {chi1R -> chi1R + 1/2 dZfR chi1R},
		Chirality -> Right},
  F[1] == {
		ClassName -> f,
		TeXName -> "f",
		TeXAntiName -> "\\bar{f}",
		SelfConjugate -> False,
		QuantumNumbers -> {Q-> Q},
		Mass -> mf,
		WeylComponents -> {psi1L, chi1R}},		
  V[1] == {
		ClassName -> A,
		TeXName -> "\\gamma",
		Renormalization -> {A[mu_] -> A[mu] + 1/2 dZ3 A[mu]},
		Mass -> 0,
		SelfConjugate -> True}};

(***** Lagrangean *****)
LGauge := -1/4 FS[A, \[Mu], \[Nu]] FS[A, \[Mu], \[Nu]]
LFermions := I psi1Lbar.sibar[mu].del[psi1L, mu] + I chi1Rbar.si[mu].del[chi1R, mu] \
				 - mf (psi1Lbar.chi1R + chi1Rbar.psi1L) \
				 + ee psi1Lbar.sibar[mu].psi1L A[mu] + ee chi1Rbar.si[mu].chi1R A[mu]
LGF := -1/2/xiA del[A[mu], mu] del[A[nu], nu]

(***** Extra information for FeynMaster *****)
M$FCeqs = {xiA->1, ee->e};
