(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
This is the FeynMaster model file for scalar QED.
Created by: Duarte Fontes
Email: duartefontes@tecnico.ulisboa.pt
Last Update: 30.07.2020
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)

(***** Parameter list *****)
M$Parameters = {
  ee == {TeXName -> "e", Renormalization -> {ee -> ee + dee}},
  lbd == {TeXName -> "\\lambda", Renormalization -> {ldb -> ldb + dldb}},
  mH == {TeXName -> "m_H", Renormalization -> {mH -> mH + dmH}},
  xiA == {TeXName -> "\\xi_A"},
  dee == {Counterterm -> True, TeXName -> "\\delta e"},
  dldb == {Counterterm -> True, TeXName -> "\\delta \\lambda"},
  dZH == {Counterterm -> True, TeXName -> "\\delta Z_H"},
  dmH == {Counterterm -> True, TeXName -> "\\delta m_H"},
  dZ3 == {Counterterm -> True, TeXName -> "\\delta Z_3"}}

(***** Gauge group list *****)
M$GaugeGroups = {
  U1EM == {
		Abelian -> True,
		GaugeBoson -> A,
		Charge -> Q,
		CouplingConstant -> ee}}

(***** Particle classes list *****)
M$ClassesDescription = {
  S[1] == {
		ClassName -> H,
		TeXName -> "H^-",
		TeXAntiName -> "H^+",		
		SelfConjugate -> False,
		Renormalization -> {H -> H + 1/2 dZH H},
		QuantumNumbers -> {Q-> -1},
		Mass -> mH},
  V[1] == {
		ClassName -> A,
		TeXName -> "\\gamma",
		Renormalization -> {A[mu_] -> A[mu] + 1/2 dZ3 A[mu]},
		SelfConjugate -> True,
		Mass -> 0}}

(***** Lagrangean parts *****)
LGauge := -1/4 FS[A, \[Mu], \[Nu]] FS[A, \[Mu], \[Nu]]
LHiggs := HC[DC[H, \[Mu]]] DC[H, \[Mu]] - mH^2 HC[H] H -1/4 lbd (HC[H] H) (HC[H] H)
LGF := -1/2/xiA del[A[mu], mu] del[A[nu], nu]

(***** Extra information for FeynMaster *****)
M$FCeqs = {xiA->1, ee->e};
